package dev.kske.chess.board;

import java.util.ArrayList;
import java.util.List;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Bishop.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Bishop extends Piece {

	public Bishop(Color color, Board board) {
		super(color, board);
	}

	@Override
	public boolean isValidMove(Move move) {
		return move.isDiagonal() && isFreePath(move);
	}

	@Override
	protected List<Move> getPseudolegalMoves(Position pos) {
		List<Move> moves = new ArrayList<>();

		// Diagonal moves to the lower right
		for (int i = pos.x + 1, j = pos.y + 1; i < 8 && j < 8; i++, j++) {
			Move move = new Move(pos, new Position(i, j));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}

		// Diagonal moves to the lower left
		for (int i = pos.x - 1, j = pos.y + 1; i >= 0 && j < 8; i--, j++) {
			Move move = new Move(pos, new Position(i, j));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}

		// Diagonal moves to the upper right
		for (int i = pos.x + 1, j = pos.y - 1; i < 8 && j >= 0; i++, j--) {
			Move move = new Move(pos, new Position(i, j));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}

		// Diagonal moves to the upper left
		for (int i = pos.x - 1, j = pos.y - 1; i >= 0 && j >= 0; i--, j--) {
			Move move = new Move(pos, new Position(i, j));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}
		return moves;
	}

	@Override
	public Type getType() { return Type.BISHOP; }
}
