package dev.kske.chess.board;

import java.util.ArrayList;
import java.util.List;

import dev.kske.chess.board.Piece.Color;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Log.java</strong><br>
 * Created: <strong>09.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Log implements Cloneable {

	private List<LoggedMove>	moves;
	private Color				activeColor;

	public Log() {
		moves		= new ArrayList<>();
		activeColor	= Color.WHITE;
	}

	public void add(Move move, Piece capturedPiece, boolean pawnMove) {
		// En passant availability
		Position enPassant = null;
		if (pawnMove && move.yDist == 2) enPassant = new Position(move.pos.x, move.pos.y + move.ySign);

		// Fullmove counter and halfmove clock
		int fullmoveCounter, halfmoveClock;
		if (moves.isEmpty()) {
			fullmoveCounter	= 1;
			halfmoveClock	= 0;
		} else {
			fullmoveCounter = getLast().fullmoveCounter;
			if (activeColor == Color.BLACK) ++fullmoveCounter;
			halfmoveClock = capturedPiece != null || pawnMove ? 0 : getLast().halfmoveClock + 1;
		}
		activeColor = activeColor.opposite();
		moves.add(new LoggedMove(move, capturedPiece, enPassant, fullmoveCounter, halfmoveClock));
	}

	public LoggedMove getLast() { return moves.isEmpty() ? null : moves.get(moves.size() - 1); }

	public void removeLast() {
		if (!moves.isEmpty()) {
			activeColor = activeColor.opposite();
			moves.remove(moves.size() - 1);
		}
	}

	public void reset() {
		moves.clear();
		activeColor = Color.WHITE;
	}

	@Override
	public Object clone() {
		Log log = null;
		try {
			log = (Log) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		log.moves = new ArrayList<>();
		log.moves.addAll(this.moves);
		return log;
	}

	public Color getActiveColor() { return activeColor; }

	public void setActiveColor(Color activeColor) { this.activeColor = activeColor; }

	public List<LoggedMove> getLoggedMoves() { return moves; }

	public static class LoggedMove {

		public final Move		move;
		public final Piece		capturedPiece;
		public final Position	enPassant;
		public final int		fullmoveCounter, halfmoveClock;

		public LoggedMove(Move move, Piece capturedPiece, Position enPassant, int fullmoveCounter, int halfmoveClock) {
			this.move				= move;
			this.capturedPiece		= capturedPiece;
			this.enPassant			= enPassant;
			this.fullmoveCounter	= fullmoveCounter;
			this.halfmoveClock		= halfmoveClock;
		}
	}
}
