package dev.kske.chess.board;

import java.util.ArrayList;
import java.util.List;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Rook.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Rook extends Piece {

	public Rook(Color color, Board board) {
		super(color, board);
	}

	@Override
	public boolean isValidMove(Move move) {
		return (move.isHorizontal() || move.isVertical()) && isFreePath(move);
	}

	@Override
	protected List<Move> getPseudolegalMoves(Position pos) {
		List<Move> moves = new ArrayList<>();

		// Horizontal moves to the right
		for (int i = pos.x + 1; i < 8; i++) {
			Move move = new Move(pos, new Position(i, pos.y));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}

		// Horizontal moves to the left
		for (int i = pos.x - 1; i >= 0; i--) {
			Move move = new Move(pos, new Position(i, pos.y));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}

		// Vertical moves to the top
		for (int i = pos.y - 1; i >= 0; i--) {
			Move move = new Move(pos, new Position(pos.x, i));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}

		// Vertical moves to the bottom
		for (int i = pos.y + 1; i < 8; i++) {
			Move move = new Move(pos, new Position(pos.x, i));
			if (board.getDest(move) == null || board.getDest(move).getColor() != getColor()) {
				moves.add(move);
				if (board.getDest(move) != null) break;
			} else break;
		}
		return moves;
	}

	@Override
	public Type getType() { return Type.ROOK; }
}
