package dev.kske.chess.ui;

import java.awt.BorderLayout;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableModel;

import dev.kske.chess.board.Log;
import dev.kske.chess.board.Log.LoggedMove;
import dev.kske.chess.board.Move;
import dev.kske.chess.event.EventBus;
import dev.kske.chess.event.Subscribe;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>LogFrame.java</strong><br>
 * Created: <strong>17.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class LogFrame extends JFrame {

	private static final long serialVersionUID = 1932671698254197119L;

	private JPanel	mcontentPane;
	private JTable	mtable;

	private Log log;

	/**
	 * Create the frame.
	 */
	public LogFrame(Log log) {
		this.log = log;

		setTitle("Move History");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 450, 300);
		mcontentPane = new JPanel();
		mcontentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		mcontentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(mcontentPane);

		mtable = new JTable();
		mtable.setEnabled(false);
		mcontentPane.add(new JScrollPane(mtable), BorderLayout.CENTER);

		EventBus.getInstance().register(this);
		EventBus.getInstance().post(new Move(0, 0, 0, 0));
	}

	@Subscribe
	public void updateData(Move move) {
		final List<LoggedMove>	moves	= log.getLoggedMoves();
		String[][]				data	= new String[moves.size() / 2 + moves.size() % 2][2];
		for (int i = 0; i < data.length; i++) {
			data[i][0] = moves.get(i * 2).move.toSAN();
			if (i * 2 + 1 < moves.size()) data[i][1] = moves.get(i * 2 + 1).move.toSAN();
		}
		mtable.setModel(new DefaultTableModel(data, new String[] { "White", "Black" }));
	}
}
